
/************************************************************
SRL1.cpp - A Library for Controlling the MRK1 Sumo Robot League robot
Author - William J Ashby, PhD <washby@sumorobotleague.com>
This library is released into the public domain.
************************************************************/
#include "Arduino.h"
#include "SRL1.h"

#define buttonPin 2
#define pingPin 3
#define echoPin A0
#define leftSensor A1
#define rightSensor A2
#define rearSensor A3

#define leftDirection 4
#define buzzerPin 5
#define IREmitter 6
#define LED 7
#define rightDirection 8
#define leftMotorSpeedPin 9
#define rightMotorSpeedPin 10


bool debug1;
bool leftIsFlipped; //If the left motor goes backwards when it is supposed to go forwards set to true
bool rightIsFlipped; 
int lThresh; //default threshold values inbetween black and white for the line sensors
int rThresh;
int rearThresh;
double tuningTurn;
double tuningFwd;
double tuningBwd;

SRL1::SRL1() {
	debug1 = false;
	leftIsFlipped = false;
    rightIsFlipped = false; 
	lThresh = 500; //default threshold values inbetween black and white for the line sensors
	rThresh = 500;
	rearThresh = 500;
	tuningTurn = 1.0;
	tuningFwd = 1.0;
	tuningBwd = 1.0;
}

void SRL1::debug(int val){
	if(val == 0) {
		debug1 = false;
	}else{
		debug1 = true;
	}
}

void SRL1::flipMotorDirection(int mtr){
	if( mtr == 0 ){
		leftIsFlipped = !leftIsFlipped;
	}
	if( mtr == 1 ){
		rightIsFlipped = !rightIsFlipped;
	}
}

void SRL1::tuneForward(double x){
	tuningFwd = x;
}

void SRL1::tuneBackward(double x){
	tuningBwd = x;
}

void SRL1::tuneTurn(double x){
	tuningTurn = x;
}

void SRL1::jumpStartSetup(){
  Serial.begin(115200);
  pinMode(pingPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(leftSensor, INPUT);
  pinMode(rightSensor, INPUT);
  pinMode(rearSensor, INPUT);  
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(buzzerPin, OUTPUT);  
  pinMode(IREmitter, OUTPUT);
  pinMode(leftMotorSpeedPin, OUTPUT);
  pinMode(rightMotorSpeedPin, OUTPUT);  
  pinMode(leftDirection, OUTPUT);
  pinMode(rightDirection, OUTPUT);
  pinMode(LED, OUTPUT);	
}

void SRL1::Forward(int amt){
  drive(80);  //turn the motors on forward 80% maximum speed
  delay(amt*50*tuningFwd);  //wait
  drive(0);  //turn off the motors
}

void SRL1::Backward(int amt){
  drive(-80);  //turn the motors on forward 80% maximum speed
  delay(amt*50*tuningBwd);  //wait
  drive(0);  //turn off the motors
}

void SRL1::Left(int deg){
  turnDegrees( -deg );
}

void SRL1::Right(int deg){
  turnDegrees( deg);
}

void SRL1::PD(){
  tone(buzzerPin, 3600, 800);
}

void SRL1::PU(){
  tone(buzzerPin, 2200, 400);
}

void SRL1::turnDegrees(int y) {
 
  if (y > 0) {
    setSpeedForMotor(200, 0);
    setSpeedForMotor(-200, 1);
 
  } else if (y < 0) {
    setSpeedForMotor(-200, 0);
    setSpeedForMotor(200, 1);
    
  } else {
	 if(debug1) Serial.println("0 degree turn");
  }
  delay( abs( (int) (tuningTurn*9*y) ) );  
  setSpeedForMotor(0,0);
  setSpeedForMotor(0,1);
}

void SRL1::drive(int percentPower) {
  // map converts [-100,100] to [-255,255]
  int spd = map(percentPower, -100, 100, -255, 255);
  setSpeedForMotor(spd, 0);
  setSpeedForMotor(spd, 1);
}

void SRL1::setSpeedForMotor(int spd, int motor) {
  int speedPin, directionPin;
  bool flippedPin;
  if ( motor == 0 ) {
    speedPin = leftMotorSpeedPin;
    directionPin = leftDirection;
    if(leftIsFlipped){
		spd = -spd;
	}
  } else if ( motor == 1 ) {
    speedPin = rightMotorSpeedPin;
    directionPin = rightDirection;
    if(!rightIsFlipped){
       spd = -spd;
    }
  }

  if ( spd == 0 ) {
    analogWrite(speedPin, 0);
    digitalWrite(directionPin, LOW);
  } else if ( spd > 0 ) {
    digitalWrite(directionPin, HIGH);
    analogWrite(speedPin, abs(spd));
  } else if ( spd < 0 ) {
    digitalWrite(directionPin, LOW);
    analogWrite(speedPin, abs(spd));
  }
}

bool SRL1::objectWithin(int cm) {
  return getDistance() < cm;
}

int SRL1::getDistance() {
  long duration;
  int distance;
  digitalWrite(pingPin, LOW);// Added this line
  delayMicroseconds(2); // Added this line
  digitalWrite(pingPin, HIGH);
  delayMicroseconds(10); // Added this line
  digitalWrite(pingPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  distance = (int) ( (duration / 2) / 29.1 );
  return distance;
}

//returns true if either front IR sensor detects white
//uses the thresholds for the left and right front IR sensors that
//are calculated via the calibrateBlackWhite function
bool SRL1::isWhite() {
  digitalWrite(IREmitter, HIGH);
  delayMicroseconds(2);
  int left = analogRead(leftSensor);
  int right = analogRead(rightSensor);
  if(debug1){
	  Serial.print("left: "); Serial.println(left);
	  Serial.print("right: "); Serial.println(right);
  }
  digitalWrite(IREmitter, LOW);
  if(debug1){
	  Serial.print("lThresh: "); Serial.println(lThresh);
	  Serial.print("rThresh: "); Serial.println(rThresh);
  }
  
  if (left < lThresh || right < rThresh) {
    return true;
  } else return false;
}

//return the threshold value for the IR line sensors
int SRL1::getThreshold(int side) {
	if( side == 15){
		return lThresh;
	}
	if( side == 16){
		return rThresh;
	}
	if( side == 17){
		return rearThresh;
	}
}

void SRL1::setThreshold(int side, int val) {
	if( side == 15){
		lThresh = val;
	}
	if( side == 16){
		rThresh = val;
	}
	if( side == 17){
		rearThresh = val;
	}
}

int SRL1::getIR(int side) {
	int returnMe;
	digitalWrite(IREmitter, HIGH);
	delayMicroseconds(2);
	returnMe = analogRead(side);
	digitalWrite(IREmitter, LOW);
	if(debug1) {
		Serial.print("IR reading: "); Serial.println(returnMe);
	}
	return returnMe;
}

//averages 5 black and 5 white IR readings for each front IR sensor
//then calculates the threshold between the balck and white readings
void SRL1::calibrateBlackWhite() {
  int n = 5;  //setup arrays to average n readings of the sensors on black and on white
  int lBlacks[n]; int rBlacks[n];  int rearBlacks[n];
  int lWhites[n]; int rWhites[n];  int rearWhites[n];

  Serial.println("\nPlace line sensors");
  Serial.println("on black and press button");
  tone(buzzerPin, 1500, 400);
  while (digitalRead(buttonPin) == 1) {
    delay(100);
  }

  digitalWrite(IREmitter, HIGH);
  for (int i = 0; i < n; i++) {
    lBlacks[i] = analogRead(leftSensor);
    rBlacks[i] = analogRead(rightSensor);
	rearBlacks[i] = analogRead(rearSensor);
  }
  digitalWrite(IREmitter, LOW);

  delay(2500);
  Serial.println("\nPlace line sensors");
  Serial.println(" on white and press button");
  tone(buzzerPin, 1800, 200);
  delay(250);
  tone(buzzerPin, 1800, 200);
  delay(250);

  while (digitalRead(buttonPin) == 1) {
    delay(100);
  }

  digitalWrite(IREmitter, HIGH);
  for (int i = 0; i < n; i++) {
    lWhites[i] = analogRead(leftSensor);
    rWhites[i] = analogRead(rightSensor);
    rearWhites[i] = analogRead(rearSensor);
  }
  digitalWrite(IREmitter, LOW);
  
  int lBlack = 0;  int rBlack = 0;  int rearBlack = 0;
  int lWhite = 0;  int rWhite = 0;  int rearWhite = 0;
  for (int i = 0; i < n; i++) {
    lBlack += lBlacks[i];
    rBlack += rBlacks[i];
	rearBlack += rearBlacks[i];
    lWhite += lWhites[i];
    rWhite += rWhites[i];
	rearWhite += rearWhites[i];
  }
  lBlack = lBlack / n; rBlack = rBlack / n;  rearBlack = rearBlack / n;
  lWhite = lWhite / n; rWhite = rWhite / n;  rearWhite = rearWhite / n;
  if(debug1){
	  Serial.print("Black averages: \nLeft "); Serial.println(lBlack); 
	  Serial.print("Right "); Serial.println(rBlack); 
	  Serial.print("Rear "); Serial.println(rearBlack);
	  Serial.print("White averages: \nLeft "); Serial.println(lWhite);
	  Serial.print("Right "); Serial.println(rWhite);
	  Serial.print("Rear "); Serial.println(rearWhite);
  }
  lThresh = lWhite/2 + lBlack/2;
  rThresh = rWhite/2 + rBlack/2;
  rearThresh = rearWhite/2 + rearBlack/2;

  delay(10);
}

